﻿using Infragistics.Olap.FlatData;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            Load += SampleForm_Load;
        }

        private void SampleForm_Load(object sender, EventArgs e)
        {
            FlatDataSource ds = InitializePivotData();
            // Generate cubes and call "InitializeAsync" method passing the control the data is bound to.
            var parameters = new CubeGenerationParameters();
            ds.GenerateCube(parameters);
            ds.InitializeAsync(ultraPivotGrid1);
            // Bind the PivotGrid and DataSelector to data.
            ultraPivotGrid1.DataSource = ds;
            olapDataSelector1.DataSource = ds;
            // Optinally, set compact mode on PivotGrid
            ultraPivotGrid1.RowHeaderLayout = Infragistics.Win.UltraWinPivotGrid.RowHeaderLayout.Compact;
        }

        public FlatDataSource InitializePivotData()
        {
            // Initialize the data
            SampleFlatData data = new SampleFlatData();
            IEnumerable<SportsData> itemSource = data.InitSportsData();
            // Use "FlatDataSourceInitialSettings" object to initialize Hierarchies and Measures
            var settings = new FlatDataSourceInitialSettings();
            settings.Rows = "[SportsData].[Date]";
            settings.Columns = "[SportsData].[Sport]";
            settings.Measures = "[Measures].[Budget]";
            // Create an instance of the FlatDataSource
            var ds = new FlatDataSource(itemSource, typeof(SportsData), settings);
            // Initialize Hierarchy Descriptor
            ds.InitializeHierarchyDescriptor += ds_InitializeHierarchyDescriptor;
            // Initialize Measure Descriptor
            ds.InitializeMeasureDescriptor += ds_InitializeMeasureDescriptor;
            // Aggregate measures to produce totals
            ds.AggregateMeasure += ds_AggregateMeasure;
            return ds;
        }

        void ds_InitializeHierarchyDescriptor(object sender, InitializeHierarchyDescriptorEventArgs e)
        {
            LevelDescriptor level;
            if (e.HierarchyDescriptor.LevelDescriptors.TryGetItem("all", out level))
            {
                switch (e.HierarchyDescriptor.PropertyDescriptor.Name)
                {
                    case "Nation":
                        level.MemberProvider = item => "All Nations";
                        break;
                    case "City":
                        level.MemberProvider = item => "All Cities";
                        break;
                    case "Sport":
                        level.MemberProvider = item => "All Sports";
                        break;
                }
            }

            if(e.HierarchyDescriptor.Name == "Date")
            {
                e.HierarchyDescriptor.LevelDescriptors.Clear();

                LevelDescriptor yearDescriptor = e.HierarchyDescriptor.LevelDescriptors.Add("year");
                yearDescriptor.Caption = "Year";
                yearDescriptor.MemberProvider = item => (item as SportsData).Date.Year.ToString();

                LevelDescriptor semesterDescriptor = e.HierarchyDescriptor.LevelDescriptors.Add("semester");
                semesterDescriptor.Caption = "Semester";
                semesterDescriptor.MemberProvider = item => semesterWithYear((item as SportsData).Date);

                LevelDescriptor quarterDescriptor = e.HierarchyDescriptor.LevelDescriptors.Add("quarter");
                quarterDescriptor.Caption = "Quarter";
                quarterDescriptor.MemberProvider = item => quarterWithYear((item as SportsData).Date);

                LevelDescriptor monthDescriptor = e.HierarchyDescriptor.LevelDescriptors.Add("month");
                monthDescriptor.Caption = "Month";
                monthDescriptor.MemberProvider = item => (item as SportsData).Date.Month.ToString();

                LevelDescriptor dayDescriptor = e.HierarchyDescriptor.LevelDescriptors.Add("day");
                dayDescriptor.Caption = "Day";
                dayDescriptor.MemberProvider = item => (item as SportsData).Date.ToShortDateString();
            }
        }
        string semesterWithYear(DateTime dateTime)
        {
            if (dateTime.Month < 7)
            {
                return "H1 - " + dateTime.Year;
            }
            else
            {
                return "H2 - " + dateTime.Year;
            }
        }

        string quarterWithYear(DateTime dateTime)
        {
            if (dateTime.Month < 4)
            {
                return "Q1 - " + dateTime.Year;
            }
            else if (dateTime.Month < 7)
            {
                return "Q2 - " + dateTime.Year;
            }
            if (dateTime.Month < 10)
            {
                return "Q3 - " + dateTime.Year;
            }
            else
            {
                return "Q4 - " + dateTime.Year;
            }
        }

        void ds_InitializeMeasureDescriptor(object sender, InitializeMeasureDescriptorEventArgs e)
        {
            if (e.MeasureDescriptor.Name == "Budget")
            {
                e.MeasureDescriptor.DisplayFormat = "$#";
                e.MeasureDescriptor.Aggregation = MeasureAggregation.Average;
            }
        }
        void ds_AggregateMeasure(object sender, AggregateMeasureEventArgs e)
        {
            if (e.MeasureDescriptor.Name == "Budget")
            {
                double total = 0f;
                double count = 0f;
                foreach (object o in e.Items)
                {
                    var item = o as SportsData;
                    if (item != null)
                        total += item.Budget;
                    count += 1f;
                }
                e.Value = total / count;
                e.Handled = true;
            }
        }
    }
}
